//+------------------------------------------------------------------+
//|                                                  Moon Phases.mq4 |
//|                                        Copyright  2008, LEGRUPO |
//|                                           http://www.legrupo.com |
//|                                                     Version: 1.0 |
//| History:                                                         |
//| 1.0 => Release version to the public                             |
//+------------------------------------------------------------------+
#property copyright "Copyright  2008, LEGRUPO"
#property link      "http://www.legrupo.com"

#property indicator_chart_window
#property indicator_buffers 2
#property indicator_color1  DarkGray
#property indicator_color2  White

//
//
//
//
//

extern string note1           = " calculates the moon phase (0-7)";
extern string note2           = " accurate to 1 segment.";
extern string note3           = " 0 = new moon and 4 = full moon";
extern int    mpA             = 0;
extern int    mpB             = 4;
extern int    dotWidth        = 4;
extern string note            = "turn on Alert = true; turn off = false";
extern bool   alertsOn        = true;
extern bool   alertsOnCurrent = true;
extern bool   alertsMessage   = true;
extern bool   alertsSound     = true;
extern bool   alertsEmail     = false;
extern string soundfile       = "alert2.wav";

//
//
//
//
//

double new_moon[];
double full_moon[];
double trend[];

//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
//
//

int init()
{
   IndicatorBuffers(3);
   SetIndexBuffer(0,new_moon); SetIndexStyle(0,DRAW_ARROW,0,dotWidth); SetIndexArrow(0,161); SetIndexLabel(0,"New Moon");
   SetIndexBuffer(1,full_moon);SetIndexStyle(1,DRAW_ARROW,0,dotWidth); SetIndexArrow(1,108); SetIndexLabel(1,"Full Moon");
   SetIndexBuffer(2,trend);
return(0);
}

//+------------------------------------------------------------------+
//| Custom indicator deinitialization function                       |
//+------------------------------------------------------------------+
//
//

int deinit()  {   return(0);  }

//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
//
//

int start()
{
   int counted_bars=IndicatorCounted();
   int i,limit;

   if(counted_bars<0) return(-1);
   if(counted_bars>0) counted_bars--;
         limit = MathMin(Bars-counted_bars,Bars-1);
         
   //
   //
   //
   //
   //
   
   for(i=limit; i>=0; i--) 
   {
      int moon_actual = moon_phase(TimeYear(Time[i]),TimeMonth(Time[i]),TimeDay(Time[i]));
         trend[i] = trend[i+1];
         if (moon_actual == mpA) trend[i] = 1;
         if (moon_actual == mpB) trend[i] =-1;
         
         //
         //
         //
         //
         //
         
         new_moon[i]  = EMPTY_VALUE;
         full_moon[i] = EMPTY_VALUE;
         if (trend[i]!= trend[i+1])
         if (trend[i] == 1)
                new_moon[i]  = High[i] + iATR(NULL,0,20,i)/2;
          else  full_moon[i] = High[i] + iATR(NULL,0,20,i)/2;       
     }
     
     //
     //
     //
     //
     //
     
     if (alertsOn)
     {
       if (alertsOnCurrent)
            int whichBar = 0;
       else     whichBar = 1;
         
       if (trend[whichBar] != trend[whichBar+1])
       if (trend[whichBar] == 1)
             doAlert("NewMoon");
       else  doAlert("FullMoon");       
   }
   
   return(0);
}

//
//
//
//
//

int moon_phase(int y, int m, int d)
{
    /*
    calculates the moon phase (0-7), accurate to 1 segment.
    0 = > new moon.
    4 => full moon.
    */

    int    c,e;
    double jd;
    int    b;

    if (m < 3) {
        y--;
        m += 12;
    }
    m = 1+m;
    c = 365.25*y;
    e = 30.6*m;
    jd = c+e+d-694039.09;  /* jd is total days elapsed */
    jd /= 29.53;           /* divide by the moon cycle (29.53 days) */
    b = jd;		            /* int(jd) -> b, take integer part of jd */
    jd -= b;		         /* subtract integer part to leave fractional part of original jd */
    b = jd*8 + 0.5;	      /* scale fraction from 0-8 and round by adding 0.5 */
    b = b & 7;		         /* 0 and 8 are the same so turn 8 into 0 */
return(b);
}

//
//
//
//
//

void doAlert(string doWhat)
{
   static string   previousAlert="nothing";
   static datetime previousTime;
   string message;
   
      if (previousAlert != doWhat || previousTime != Time[0]) {
          previousAlert  = doWhat;
          previousTime   = Time[0];

          //
          //
          //
          //
          //

          message =  StringConcatenate(Symbol()," at ",TimeToStr(TimeLocal(),TIME_SECONDS)," Moon Phases ",doWhat);
             if (alertsMessage) Alert(message);
             if (alertsEmail)   SendMail(StringConcatenate(Symbol()," Moon Phases "),message);
             if (alertsSound)   PlaySound(soundfile);
      }
}

